/*
  Taken from 
  Cone3D GFX with SDL lesson 5.
  Made by Marius Andra 2002
  http://cone3d.gamedev.net

  Converted to Dingoo
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>

#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"
#include "font.h"

//Audio stuff
Mix_Music *music = NULL;
Mix_Chunk *pop = NULL;
int audio_rate = 22050;
Uint16 audio_format = AUDIO_S16; /* 16-bit stereo */
int audio_channels = 2;
int audio_buffers = 1024;



// These two variables contain the dimensions of the playfield
int rows;
int cols;

SDL_Surface *screen;   // The screen surface
SDL_Surface *balls[10];// The ball images
SDL_Surface *background;
SDLFont *font1;        // 2 fonts
SDLFont *font2;


char *playf;           // The playfield itself

// The default width and height of the screen
int scrwidth=320, scrheight=240;
int centx=0,centy=0;   // Used to center the grid nicely onto the screen

int bls=4;             // Number of differently colored balls

int bla=0;             // Used with the mouse...

int score=0;           // The current score
int bestScore=0;
int ballGap = 21;
int done =0;
// This function draws a surface onto the screen
void DrawIMG(SDL_Surface *img, int x, int y)
{
  SDL_Rect dest;
  dest.x = x;
  dest.y = y;
  SDL_BlitSurface(img, NULL, screen, &dest);
}

// This swaps two type char variables
void swap(char &r1, char &r2)
{
  char temp=r1;
  r1=r2;
  r2=temp;
}

// Returns the color of a ball in the playfield.
inline char &grid(int a, int b)
{
  return playf[a*cols+b];
}

// This function makes balls fall down and rows move left.
void collapse()
{
  // Make balls fall down
  int to=-1;
  for(int j=0;j<cols;j++)
  {
    to=-1;
    for(int i=rows-1;i>=0;i--)
    {
      if(to==-1 && grid(i,j)==-1) {to=i;}
      else if(to!=-1 && grid(i,j)!=-1) {swap(grid(i,j),grid(to,j)); to--;}
    }
  }
  
  // Move rows to the left
  to=-1;
  {
    for(int j=0;j<cols;j++)
    {
  	  if(to==-1 && grid(rows-1,j)==-1)
      {
        to=j;
      } else if(to!=-1 && grid(rows-1,j)!=-1) {
        for(int i=0;i<rows;i++)
        {
          grid(i,to) = grid(i,j);
        }
  	    to++;
      }
    }
	
    {
      if(to!=-1)
      {
        for(int j=to;j<cols;j++)
        {
          for(int i=0;i<rows;i++)
          {
            grid(i,j)=-1;
          }
   	    }
      }
    }
  }
}

// This is a recursive function that clears a bunch of balls.
// It uses the flood fill algorithm
int pick(int i, int j, int a)
{
  int sum=1;
  grid(i,j)=-1;
  if(i != 0 && grid(i-1,j)==a) sum+=pick(i-1,j,a);
  if(j != 0 && grid(i,j-1)==a) sum+=pick(i,j-1,a);
  if(i<rows-1 && grid(i+1,j) == a) sum+=pick(i+1,j,a);
  if(j<cols-1 && grid(i,j+1) == a) sum+=pick(i,j+1,a);
  return sum;
}

// This function checks whether one ball is connected to an other
int bunch(int i,int j)
{
  int a = grid(i,j);
  if(i!=0 && grid(i-1,j) == a) return 1;
  if(j!=0 && grid(i,j-1) == a) return 1;
  if(i<rows-1 && grid(i+1,j) == a) return 1;
  if(j<cols-1 && grid(i,j+1) == a) return 1;
  return 0;
}

// Draw the screen
void DrawScene()
{
  SDL_ShowCursor(0);                    // Hide the mouse cursor
  //SDL_FillRect(screen,NULL,0x000000);   // Clear the entire screen with black
   SDL_BlitSurface(background, NULL, screen, NULL);
	
  // Draw the balls
  for(int i=0;i<rows;i++)
  {
    for(int j=0;j<cols;j++)
    {
      if(grid(i,j)!=-1) DrawIMG(balls[grid(i,j)],j*ballGap+centx,i*ballGap+centy);
    }
  }

  // Draw some info and the score
  drawString(screen,font2,1,scrheight-16,"Score: %d    Best: %d ",score,bestScore);

  
  //SDL_Flip(screen);        // Flip the buffers
  SDL_UpdateRect(screen,0,0,0,0);
  SDL_ShowCursor(1);       // Show the cursor again
}

// This function resets the entire grid by adding random balls to it
void newgame()
{
  // Reset the score
  score=0;

  // Fill the grid with random balls
  for(int i=0;i<rows;i++)
  {
    for(int j=0;j<cols;j++)
    {
      grid(i,j) = rand()%bls;
    }
  }

  // Redraw the screen
  DrawScene();
}


void drawStringCentered(SDLFont *font, int y, char *str)
{
	int w = stringWidth(font2,str);
	drawString(screen,font,(scrwidth-w)/2,y,str);
}

void TitleScreen(){
    
    
     SDL_Event event;
     float ay=0.1;
     int y=0;	
     while (1)	
     {
	y=y+ay;
	ay = ay + 0.1;
	 if (ay > 3)  { ay=-ay; }  
	  SDL_BlitSurface(background, NULL, screen, NULL);
	drawStringCentered(font2,50+y,"Dingoo Balls");
	drawStringCentered(font1,70+y,"The more you pop the more you score");
	drawStringCentered(font1,90+y,"A= Pop, X = Start Again, SELECT = quit");
	drawStringCentered(font1,110+y,"Press any button to start");
	SDL_UpdateRect(screen,0,0,0,0);
	SDL_Delay(10);
	SDL_PollEvent(&event);
          // If someone closes the prorgam, then quit
	if ( event.type == SDL_QUIT )  {  done = 1;  break ;}
	if ( event.type == SDL_KEYDOWN) { break; }
      }
      
}




// This is our main function
int main(int argc, char *argv[])
{
  srand(time(NULL));    // We initalize the random number generator

 
  // Initalize SDL
  if ( SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0 )
  {
    printf("Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);

  // We load in a random ball image as the icon for this program
  char tempstring[100];
  sprintf(tempstring,"data/balls/%d.bmp",rand()%9+1);
  SDL_WM_SetIcon(SDL_LoadBMP(tempstring),NULL);

  // Initalize the video mode
  screen=SDL_SetVideoMode(scrwidth,scrheight,16,SDL_SWSURFACE|SDL_HWPALETTE);
  if ( screen == NULL )
  {
    printf("Unable to set %dx%d video: %s\n", scrwidth, scrheight, SDL_GetError());
    exit(1);
  }

  if(Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers)) {
    printf("Unable to open audio!\n");
    exit(1);
  }

  
  
  // Load in the fonts
  font1 = initFont("data/font1");
  font2 = initFont("data/font2",1,1,0);

  // Load in all the balls
  for(int a=1;a<5;a++)
  {
    char temp[100];
    sprintf(temp,"data/balls/%d.bmp",a);
    balls[a-1]=SDL_LoadBMP(temp);
    SDL_SetColorKey( balls[a-1], SDL_SRCCOLORKEY, SDL_MapRGB(balls[a-1]->format, 0, 0, 0) );  
  }

  background = SDL_LoadBMP("data/background.bmp");
  // Calculate how much balls fit on the screen with the current
  // screen resolution
  rows=(scrheight-16)/ballGap;
  cols=(scrwidth)/ballGap;

  // Calculate how much must the balls be movedso that they would be centered.
  centx = (scrwidth-cols*ballGap)/2;
  centy = (scrheight-16-rows*ballGap)/2;

  // Allocate space for the playfield
  playf = new char[rows*cols];

  //run title
  TitleScreen();
  
  // Reset the score and generate the playfield
  newgame();
  int lastTick=0;
  int dx=0;
  int dy=0;
  int click=0;
  float acl=0;
  // Loop a bit
  done=0;
  pop = Mix_LoadWAV("data/sounds/pop.wav");
  music = Mix_LoadMUS("data/sounds/music.mod");
  Mix_Volume(-1, 64);
  Mix_PlayMusic(music, 0);
   
  while(done == 0)
  {
	  
	  
   	  
    SDL_Event event;

    while ( SDL_PollEvent(&event) )
    {
      // If someone closes the prorgam, then quit
      if ( event.type == SDL_QUIT )  {  done = 1;  }

      if ( event.type == SDL_KEYDOWN )
      {
        // If someone presses ESC, then quit
        if ( event.key.keysym.sym == SDLK_ESCAPE ) { done = 1; }
        // Space restarts the game
        if ( event.key.keysym.sym == SDLK_SPACE ) { newgame(); }
        // The gray keypad + increases the nr. of different balls
        if ( event.key.keysym.sym == SDLK_KP_PLUS ) { if(bls<9) {bls++;newgame();} }
        // The gray keypad - decreases the nr. of different balls
        if ( event.key.keysym.sym == SDLK_KP_MINUS ) { if(bls>3) {bls--;newgame(); } }
	//mouse fake on the dingoo
	if ( event.key.keysym.sym == SDLK_DOWN ) {
		dy=2;
	}
	if ( event.key.keysym.sym == SDLK_UP ) {
		dy=-2;
	}
	if ( event.key.keysym.sym == SDLK_LEFT ) {
		dx=-2;
	}
	if ( event.key.keysym.sym == SDLK_RIGHT ) {
		dx=2;
	}
	if ( event.key.keysym.sym == SDLK_LCTRL ) {
		click = 1;
	}
	
      }
      if ( event.type == SDL_KEYUP ) {
	      acl=0; bla= 1;
	 if ( event.key.keysym.sym == SDLK_DOWN ) {
		dy=0;
	}
	if ( event.key.keysym.sym == SDLK_UP ) {
		dy=0;
	}
	if ( event.key.keysym.sym == SDLK_LEFT ) {
		dx=0;
	}
	if ( event.key.keysym.sym == SDLK_RIGHT ) {
		dx=0;
	}     
	      
      }
    }

    int x,y;     // Used to hold the mouse coordinates
    int x2,y2;   // Used to hold the ball coordinates
    float x3,y3; // Used to temporarily hold the ball coordinates
    SDL_GetMouseState(&x, &y);    // Get the mouse coords
    if ((dy != 0 || dx!= 0) && ((SDL_GetTicks() - lastTick) > 30)) { //woh, not too fast please
	SDL_WarpMouse(x+dx*acl,y+dy*acl);
	SDL_UpdateRect(screen,0,0,0,0);
	acl += 0.2;
	lastTick = SDL_GetTicks();
    }
    
    x3=(((float)(x-centx))/ballGap);   // Get the ball coords
    y3=(((float)(y-centy))/ballGap);
    if(x3<0 || y3<0) continue;    // If they are negative, restart the loop
    else {x2=(int)x3;y2=(int)y3;} // else store them as ints

    // If the mouse button is released, make bla equal 1
    if(!SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(1))
    {
      bla=1;
    }

    // If the mouse button was released the previous frame and is now
    // held down, then ...
    if ((SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(1) && bla==1) || click==1)
    {
      // Make bla equal zero so that we could get here only if we release
      // the mouse button and then click it again.
	       
      bla=0;
      click=0;
      // Investigate the clicked ball
      if(x2>=0 && y2>=0 && x2<cols && y2<rows && grid(y2,x2) != -1 && bunch(y2,x2))
      {
        // If it really is clickable then get rid of the balls
        int a=pick(y2,x2,grid(y2,x2));
        // Increase the score by the (removed balls-2)^2
        score+=(a-2)*(a-2);
	if (score > bestScore)  { bestScore = score ; }   
        // Make the balls fall down
	 Mix_PlayChannel(-1, pop, 0);  
        collapse();
        // And update the screen
        DrawScene();
	      
      }
    }
  }

  // Let's clean up...
  Mix_CloseAudio();
  freeFont(font1);
  freeFont(font2);

  return 0;
}

